import 'package:flutter/material.dart';

/// Types of animations
enum AnimationType { normal, fadeIn }

/// Main class, [context] BuildContext application
/// [child] navigate widget, [animation] type animation
class RouteTransitions {
  final BuildContext context;
  final Widget child;
  final AnimationType animation;
  final Duration duration;
  final bool replacement;

  RouteTransitions(
      {@required this.context,
      @required this.child,
      this.replacement = false,
      this.duration = const Duration(milliseconds: 300),
      this.animation = AnimationType.normal}) {
    switch (this.animation) {
      case AnimationType.normal:
        this._normalTransition();
        break;
      case AnimationType.fadeIn:
        this._fadeInTransition();
        break;
    }
  }

  /// Normal transition
  void _normalTransition() {
    final route = MaterialPageRoute(builder: (_) => child);
    (replacement) ? _pushReplacementPage(route) : _pushPage(route);
  }

  /// Fade in transition
  void _fadeInTransition() {
    final route = PageRouteBuilder(
      pageBuilder: (context, animation, secondaryAnimation) => this.child,
      transitionDuration: duration,
      transitionsBuilder: (context, animation, secondaryAnimation, child) {
        return FadeTransition(
          child: child,
          opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
              CurvedAnimation(parent: animation, curve: Curves.easeOut)),
        );
      },
    );

    (replacement) ? _pushReplacementPage(route) : _pushPage(route);
  }

  /// Normal push page
  void _pushPage(Route route) => Navigator.push(context, route);

  /// Replacement push page
  void _pushReplacementPage(Route route) =>
      Navigator.pushReplacement(context, route);
}
